package engine;

import strategie.Strategie;

public class Engine
{
    /**
     * Aktuelle Situation.
     * 0==nicht besetzt
     * 1==von Spieler 1 besetzt
     * 2==von Spieler 2 besetzt
     * 3==gesperrt ;-)
     */
    private int[] feld;
    private int dran;
    
    private Strategie strategie;
    
    public void initialisieren()
    {
        feld = new int[9];
        dran=1;
        strategie = new Strategie();
    }
    
    public Engine()
    {
        initialisieren();
    }
    
    public boolean pruefeVoll()
    {
        return strategie.voll(feld);
    }
    
    public int pruefeGewonnen()
    {
        return strategie.gewonnen(feld);
    }
    
    /**
     * Prüfen, ob ein gewünschter Zug erlaubt ist.
     */
    public boolean erlaubt(int zug)
    {
        if(zug<0 || zug>8)
            return false;
        if(feld[zug]!=0)    
            return false;
        return true;    
    }
    
    public boolean gesperrt(int zug)
    {
        if(feld[zug]==3)
            return true;
        else
            return false;
    }
    
    /**
     * man übergibt Koordinaten im Spielfeld und bekommt den Zustand an dieser Stelle.<br/>
     * Index beginnend mit 0, also zeile == 0..2, spalte==0..2
     */
    public int holeZustand(int zeile, int spalte) 
    {
        return feld[zeile*3+spalte];
    }
    
    /**
     * Benutzer setzt.
     * Liefert false, wenn dieser Vorgang erfolglos ist.
     */
    public boolean setzeManuell(int zeile,int spalte)
    {
        if(zeile<0 || zeile>2 || spalte<0 || spalte>2)
             return false;
        if(feld[zeile*3+spalte]!=0)     
             return false;
             
        feld[zeile*3+spalte]=dran;
        
        if(dran==1)
            dran=2;
        else
            dran=1;
        return true;    
    }
    
    /**
     * Computer setzt an einer zufälligen Stelle.
     * Liefert false, wenn dieser Vorgang erfolglos ist.
     */
    public boolean setzeAutomatischZufaellig()
    {
        int zug = strategie.zufallsSpielzug(feld);
        if(zug>=0 && zug<=8)
        {
            feld[zug]=dran;
            if(dran==1)
                dran=2;
            else
                dran=1;
            return true;                
        }
        else
        {
            return false;
        }
    }
    /**
     * Computer setzt an der günstigsten Stelle.
     * Gibt mehrere gleich gute beste Stellen, wird eine von diesen zufällig gewählt.
     * Liefert false, wenn dieser Vorgang erfolglos ist.
     */
    public boolean setzeAutomatischOptimal()
    {
        int[] neu = strategie.empfehleSpielzug(feld,dran);
        if(neu[3]>=0)
        {
            feld[neu[3]]=dran;
            if(dran==1)
                dran=2;
            else
                dran=1;
            return true;                
        }
        else
        {
            return false;
        }
    }
    
    /**
     * Für eine Fake-Anwendung, bei der der Computer für den benutzer günstige 
     * Plätze sperrt ;O)
     */
    public boolean sperreStelle(int zeile, int spalte)
    {
        if(zeile<0 || zeile>2 || spalte<0 || spalte>2)
             return false;
        if(feld[zeile*3+spalte]!=0)     
             return false;
        feld[zeile*3+spalte]=3;
        return true;
    }
    public boolean entsperreStelle(int zeile, int spalte)
    {
        if(zeile<0 || zeile>2 || spalte<0 || spalte>2)
             return false;
        if(feld[zeile*3+spalte]!=3)     
             return false;
        feld[zeile*3+spalte]=0;
        return true;
    }
    public void entsperreAlles()
    {
        for(int i=0;i<feld.length;i++)
            if(feld[i]==3)
                feld[i]=0;
    }
}
