package strategie;


import java.util.Random;

/**
*   Erweitertes Ziel: Jeder Rekursionsaufruf soll zurückgeben,
*   wieviele Spielausgänge es aus der aktuellen Situation heraus gibt und
*   in wievielen Fällen 1 bzw. 2 gewinnt.
*
*   D.h. die Funktion rekursion gibt jetzt ein Array mit 3 Plätzen zurück:
*   Index 0 : wieviele Spielausgänge es gibt
*   Index 1 : wie oft 1 gewinnt
*   Index 2 : wie oft 2 gewinnt
*
*   Benutzt werden kann "Strategie2" von "Engine" nun, indem 
*   "Strategie2" entweder einfach die jeweils aktuell möglichen Spielzüge
*   bewertet, also durch aufrufen von rekursion(..),
*   oder indem Stragegie2 einen Spielzug empfielt: empfehleSpielzug(..), s.u.
*
*/

public class Strategie
{
    private Random zufall = new Random(System.currentTimeMillis());

    public int anzahl_spiele = 0;

    public boolean voll(int[] f)
    {
        for(int i=0;i<f.length;i++)
        {
            if(f[i]==0)
            {
                return false;
            }
        }
        return true;
    }

    public int gewonnen(int[] f)
    {
        if(f[0]==1 && f[1]==1 && f[2]==1) return 1;
        if(f[3]==1 && f[4]==1 && f[5]==1) return 1;
        if(f[6]==1 && f[7]==1 && f[8]==1) return 1;
        if(f[0]==1 && f[3]==1 && f[6]==1) return 1;
        if(f[1]==1 && f[4]==1 && f[7]==1) return 1;
        if(f[2]==1 && f[5]==1 && f[8]==1) return 1;
        if(f[0]==1 && f[4]==1 && f[8]==1) return 1;
        if(f[2]==1 && f[4]==1 && f[6]==1) return 1;

        if(f[0]==2 && f[1]==2 && f[2]==2) return 2;
        if(f[3]==2 && f[4]==2 && f[5]==2) return 2;
        if(f[6]==2 && f[7]==2 && f[8]==2) return 2;
        if(f[0]==2 && f[3]==2 && f[6]==2) return 2;
        if(f[1]==2 && f[4]==2 && f[7]==2) return 2;
        if(f[2]==2 && f[5]==2 && f[8]==2) return 2;
        if(f[0]==2 && f[4]==2 && f[8]==2) return 2;
        if(f[2]==2 && f[4]==2 && f[6]==2) return 2;

        return 0;
    }

    /**
    *  Jetzt mit Rückgabe der Vorhersage, wie viele Spielausgänge und wie oft 1 bzw. 2 gewinnt.
    */
    public int[] rekursion(int[] feld, int dran)  //dran == 1 oder ==2
    {
        if(voll(feld)) //Abbruchbedingung 1
        {
            anzahl_spiele++;            
            return new int[] {1,0,0}; //1 mögliches Spielende, unentschieden
        }
        if(gewonnen(feld)>0) //Abbruchbedingung 2      
        {
            anzahl_spiele++;
            if(gewonnen(feld)==1)
                return new int[] {1,1,0}; //1 mögliches Spielende 1 gewinnt
            else
                return new int[] {1,0,1}; //1 mögliches Spielende 2 gewinnt
        }

        int[] a_b_gewonnen = new int[3];

        if(dran==1) 
            dran=2;
        else
            dran=1;

        for(int i=0;i<feld.length;i++)
        {
            if(feld[i]==0)
            {
                 //Rekursionsaufruf:
                 int[] neufeld = new int[feld.length]; 
                 for(int k=0;k<neufeld.length;k++)
                     neufeld[k]=feld[k]; //aktuelle Spielsituation in neues Array kopieren
                 neufeld[i]=dran; //Neue Spielsituation eintragen               
                 int[] a_b_gewonnen_aktuell = rekursion(neufeld,dran);

                 //Statistiken zusammenführen:
                 //Durch diesen "Trick" werden alle zukünftigen Verläufe aufaddiert.
                 a_b_gewonnen[0]+=a_b_gewonnen_aktuell[0];
                 a_b_gewonnen[1]+=a_b_gewonnen_aktuell[1];
                 a_b_gewonnen[2]+=a_b_gewonnen_aktuell[2];
            }
        }                

        return a_b_gewonnen;
    }

    /**
    * Liefert ein Array zurück mit folgendem Inhalt:
    *   Index 0 : wieviele Spielausgänge es gibt
    *   Index 1 : wie oft 1 gewinnt
    *   Index 2 : wie oft 2 gewinnt
    *   Index 3 : zugehöriger empfohlener Platz 0..8, der idealerweise gesetzt werden sollte,
    *             oder -1, wenn feld bereits voll war.
    */
    public int[] empfehleSpielzug(int[] feld, int dran)
    {
        int[] empfehlung = {0,0,0,-1};
        int[] testfeld = new int[feld.length];
        int[] testfeld_gegner = new int[feld.length]; //testweise Gegner an gleicher Stelle setzen lassen.

        if(voll(feld))
            return empfehlung;
        if(gewonnen(feld)>0)
            return empfehlung;

        for(int i=0;i<feld.length;i++)
        {
            if(feld[i]==0)
            {
                for(int k=0;k<testfeld.length;k++)
                    testfeld[k]=feld[k];
                for(int k=0;k<testfeld.length;k++)
                    testfeld_gegner[k]=feld[k];
                testfeld[i]=dran;
                int neudran;
                if(dran==1)
                    neudran=2;
                else
                    neudran=1;                    
                testfeld_gegner[i]=neudran;
                
                int[] neu = rekursion(testfeld, neudran);
                
                //Neu: auch prüfen, ob aktueller Zug, wenn durch Gegner ausgeführt
                //diesen gewinnen ließe.
                //Dann unbedingt blockieren:
                //Andererseits auch ein Feld sofort belegen, das zum Gewinnen führt:
                int gegner_gewonnen = gewonnen(testfeld_gegner);
                int selber_gewonnen = gewonnen(testfeld);
                
                if(selber_gewonnen==dran)
                {
                        empfehlung[0]=1;
                        empfehlung[dran]=2;
                        empfehlung[neudran]=0;
                        empfehlung[3]=i;
                        return empfehlung;
                }                    
                else if(empfehlung[3]==-1) 
                {
                    empfehlung[0]=neu[0];
                    empfehlung[1]=neu[1];
                    empfehlung[2]=neu[2];
                    empfehlung[3]=i;
                }
                else
                {
                    double pneu = (double)neu[dran]/(double)neu[0];
                    double palt = (double)empfehlung[dran]/(double)empfehlung[0];
                    
                    if(gegner_gewonnen==neudran)
                    {
                        empfehlung[0]=1;
                        empfehlung[dran]=2;
                        empfehlung[neudran]=0;
                        empfehlung[3]=i;
                    }                    
                    else if(pneu>palt)
                    {
                        empfehlung[0]=neu[0];
                        empfehlung[1]=neu[1];
                        empfehlung[2]=neu[2];
                        empfehlung[3]=i;
                    }
                    else if(pneu==palt && zufall.nextBoolean())
                    {
                        empfehlung[0]=neu[0];
                        empfehlung[1]=neu[1];
                        empfehlung[2]=neu[2];
                        empfehlung[3]=i;
                    }
                }
            }
        }

        return empfehlung;
    }

    /**
    * Liefert eine zufällig gewählte Stelle (0..8), die noch frei ist, oder -1, wenn es keine gibt.
    */
    public int zufallsSpielzug(int[] feld)
    {
        int start = zufall.nextInt(9);
        for(int i=0;i<feld.length;i++)
            if(feld[(i+start)%feld.length]==0)        
                return (i+start)%feld.length;
        return -1;
    }

    public void zeigeSpielsituation(int[] feld, int dran)
    {
        if(dran>0)
            System.out.println("\nSpielsituation, "+dran+" ist dran:");
        for(int i=0;i<3;i++)
        {
            for(int k=0;k<3;k++)
            {
                System.out.print(feld[i*3+k]);
            }
            System.out.println();
        }
        System.out.println();
    }

    public static void main(String[] args)
    {
        Strategie strategie = new Strategie();
        int[] feld = new int[9];
        int[] a_b_gewonnen = strategie.rekursion(feld,1);
        System.out.println("Anzahl der Spiele:"+strategie.anzahl_spiele);
        System.out.println("Anzahl der Spiele:"+a_b_gewonnen[0]); //muss identisch sein!
        System.out.println("Varianten, in denen 1 gewinnt:"+a_b_gewonnen[1]); 
        System.out.println("Varianten, in denen 2 gewinnt:"+a_b_gewonnen[2]); 
        System.out.println("Unentschieden:"+(a_b_gewonnen[0]-a_b_gewonnen[1]-a_b_gewonnen[2])); 

        System.out.println();

        //Beispielsituation, für die eine Statistik erstellt wird:
        int[] testfeld = new int[] {
                                       1,0,0,
                                       0,2,0,
                                       0,1,2
                                   };
        a_b_gewonnen = strategie.rekursion(testfeld,1);
        System.out.println("Anzahl der Spiele:"+a_b_gewonnen[0]); 
        System.out.println("Varianten, in denen 1 gewinnt:"+a_b_gewonnen[1]); 
        System.out.println("Varianten, in denen 2 gewinnt:"+a_b_gewonnen[2]); 
        System.out.println("Unentschieden:"+(a_b_gewonnen[0]-a_b_gewonnen[1]-a_b_gewonnen[2])); 

        System.out.println("\n\nIm folgenden wird ein Spielverlauf durchgeführt: 1 spielt zufällig, 2 optimal, 1 beginnt:\n\n");

        int gewonnen = 0;
        boolean voll = false;
        int dran = 1;
        int[] FELD = new int[9];
        while(gewonnen==0 && voll==false)
        {
            strategie.zeigeSpielsituation(FELD,dran);

            if(dran==1)
            {
                int zug = strategie.zufallsSpielzug(FELD);
                if(zug>=0)
                    FELD[zug]=dran;

                dran=2;
            }
            else
            {
                int[] neu = strategie.empfehleSpielzug(FELD,dran);
                if(neu[3]>=0)
                    FELD[neu[3]]=dran;

                dran=1;
            }
            gewonnen = strategie.gewonnen(FELD);
            voll = strategie.voll(FELD);
        }
        if(voll==true)
        {
            System.out.println("Das Spiel ging unentschieden aus.");
        }
        else if(gewonnen==1)
        {
            System.out.println("1 hat gewonnen.");
        }
        else if(gewonnen==2)
        {
            System.out.println("2 hat gewonnen.");
        }
        strategie.zeigeSpielsituation(FELD,0);
    }
}


