package engine;

import java.util.Random;

import strategie.Strategie;

public class Engine
{
    /**
     * Aktuelle Situation.
     * 0==nicht besetzt
     * 1==von Spieler 1 besetzt
     * 2==von Spieler 2 besetzt
     * 3==gesperrt ;-)
     */
    private int[] feld;
    private int dran;
    
    private Random zufall = new Random(System.currentTimeMillis());
    
    private Strategie strategie;
    
    public void initialisieren()
    {
        feld = new int[9];
        dran=1;
        strategie = new Strategie();
    }
    
    public Engine()
    {
        initialisieren();
    }
    
    public boolean pruefeVoll()
    {
        return strategie.voll(feld);
    }
    
    public int pruefeGewonnen()
    {
        return strategie.gewonnen(feld);
    }
    
    /**
     * Prüfen, ob ein gewünschter Zug erlaubt ist.
     */
    public boolean erlaubt(int zug)
    {
        if(zug<0 || zug>8)
            return false;
        if(feld[zug]!=0)    
            return false;
        return true;    
    }
    
    public boolean gesperrt(int zug)
    {
        if(feld[zug]==3)
            return true;
        else
            return false;
    }
    
    /**
     * man übergibt Koordinaten im Spielfeld und bekommt den Zustand an dieser Stelle.<br/>
     * Index beginnend mit 0, also zeile == 0..2, spalte==0..2
     */
    public int holeZustand(int zeile, int spalte) 
    {
        return feld[zeile*3+spalte];
    }
    
    /**
     * Benutzer setzt.
     * Liefert false, wenn dieser Vorgang erfolglos ist.
     */
    public boolean setzeManuell(int zeile,int spalte)
    {
        if(zeile<0 || zeile>2 || spalte<0 || spalte>2)
             return false;
        if(feld[zeile*3+spalte]!=0)     
             return false;
        if(pruefeVoll())       //ungültige Züge vermeiden. 
            return false;
        if(pruefeGewonnen()>0) //ungültige Züge vermeiden.   
            return false;
             
             
        feld[zeile*3+spalte]=dran;
        
        if(dran==1)
            dran=2;
        else
            dran=1;
        return true;    
    }
    
    /**
     * Computer setzt an einer zufälligen Stelle.
     * Liefert false, wenn dieser Vorgang erfolglos ist.
     */
    public boolean setzeAutomatischZufaellig()
    {
        int zug = strategie.zufallsSpielzug(feld);
        if(zug>=0 && zug<=8)
        {
            feld[zug]=dran;
            if(dran==1)
                dran=2;
            else
                dran=1;
            return true;                
        }
        else
        {
            return false;
        }
    }
    /**
     * Computer setzt an der günstigsten Stelle.
     * Gibt mehrere gleich gute beste Stellen, wird eine von diesen zufällig gewählt.
     * Liefert false, wenn dieser Vorgang erfolglos ist.
     */
    public boolean setzeAutomatischOptimal(int SPIELANZAHL)
    {
        entsperreAlles();
        int[] neu = strategie.empfehleSpielzug(feld,dran);
        if(neu[3]>=0)
        {
            int stelle = strategie.zufallsSpielzug(feld);
            int frei = 0;
            for(int p=0;p<feld.length;p++)
                if(feld[p]==0)
                    frei++;
            //darf kein Feld sein, bei dem Gegner gewinnen würde, denn sonst
            //kann er eine Zwickmühle aufbauen:
            //++++
            int[] testfeld = new int[feld.length];
            int[] testfeld2 = new int[feld.length];
            for(int p=0;p<feld.length;p++)
                testfeld[p]=feld[p];
            for(int p=0;p<feld.length;p++)
                testfeld2[p]=feld[p];
            int nichtdran;
            if(dran==1)
                nichtdran=2;
            else
                nichtdran=1;
            testfeld[neu[3]]=nichtdran;
            testfeld2[neu[3]]=dran;
            int gewonnen = strategie.gewonnen(testfeld);
            int selber_gewonnen = strategie.gewonnen(testfeld2);
            
            //++++
            
            if(SPIELANZAHL==0 || zufall.nextBoolean() || stelle<0 || frei==1 || gewonnen==nichtdran || selber_gewonnen==dran)
            {
                feld[neu[3]]=dran;
            }
            else
            {
                feld[neu[3]]=3; //SPERREN!!!!!
                //anderes nehmen:
                feld[stelle]=dran;
            }
            
            if(dran==1)
                dran=2;
            else
                dran=1;
            return true;                
        }
        else
        {
            return false;
        }
    }

    public int findeOptimaleStelle()
    {
        int[] neu = strategie.empfehleSpielzug(feld,dran);
        if(neu[3]>=0)
        {
            return neu[3];
        }
        else
        {
            return -1;
        }
    }
    
    
    /**
     * Für eine Fake-Anwendung, bei der der Computer für den benutzer günstige 
     * Plätze sperrt ;O)
     */
    public boolean sperreStelle(int zeile, int spalte)
    {
        if(zeile<0 || zeile>2 || spalte<0 || spalte>2)
             return false;
        if(feld[zeile*3+spalte]!=0)     
             return false;
        feld[zeile*3+spalte]=3;
        return true;
    }
    public boolean sperreStelle(int stelle)
    {
        if(stelle<0 || stelle>8)
             return false;
        if(feld[stelle]!=0)     
             return false;
        feld[stelle]=3;
        return true;
    }
    public boolean entsperreStelle(int zeile, int spalte)
    {
        if(zeile<0 || zeile>2 || spalte<0 || spalte>2)
             return false;
        if(feld[zeile*3+spalte]!=3)     
             return false;
        feld[zeile*3+spalte]=0;
        return true;
    }
    public void entsperreAlles()
    {
        for(int i=0;i<feld.length;i++)
            if(feld[i]==3)
                feld[i]=0;
    }
    
    public boolean istGesperrt(int zeile, int spalte)
    {
        if(feld[zeile*3+spalte]==3)
            return true;
        else
            return false;
    }
    
    public double berechneChance()
    {
        return strategie.berechneChance(feld,dran);
    }
}
